from __future__ import unicode_literals
from helium import Label
from operator import attrgetter


def display_map(cls, client):
    def _count_sensor(self):
        return len(self.sensors(use_included=True))

    dict = super(Label, cls).display_map(client)
    dict.update([
        ('sensors', _count_sensor),
        ('name', attrgetter('name')),
    ])
    return dict

Label.display_map = classmethod(display_map)



import click

from itertools import islice
from helium_commander import Client, DataPoint
from helium_commander import JSONParamType
from contextlib import closing

pass_client = click.make_pass_decorator(Client)


def cli(cls, singleton=False):
    group = click.Group(name='timeseries',
                        short_help="Commands on timeseries readings.")
    resource_type = cls._resource_type()
    id_required = not singleton

    def _fetch_resource(client, id, **kwargs):
        if singleton:
            resource = cls.singleton(client)
        else:
            mac = kwargs.pop('mac', False)
            resource = cls.lookup(client, id, mac=mac)
        return resource

    @group.command('list')
    @click.argument('id', metavar=resource_type, required=id_required)
    @list_options()
    @click.option('--count', default=20,
                  help="the number of readings to fetch. Use -1 for all")
    @pass_client
    def _list(client, id, **kwargs):
        """Get timeseries readings."""
        count = kwargs.pop('count', 20)
        resource = _fetch_resource(client, id, **kwargs)
        timeseries = resource.timeseries(**kwargs)
        if count >= 0:
            timeseries = islice(timeseries, count)
        DataPoint.display(client, timeseries, **kwargs)

    @group.command('post')
    @click.argument('id', metavar=resource_type, required=id_required)
    @post_options()
    @pass_client
    def _post(client, id, **kwargs):
        """Post timeseries readings."""
        resource = _fetch_resource(client, id, **kwargs)
        timeseries = resource.timeseries()
        point = timeseries.post(**kwargs)
        DataPoint.display(client, [point], **kwargs)

    @group.command('live')
    @click.argument('id', metavar=resource_type, required=id_required)
    @list_options()
    @pass_client
    def _live(client, id, **kwargs):
        """Get live timeseries readings"""
        resource = _fetch_resource(client, id, **kwargs)
        timeseries = resource.timeseries(**kwargs)
        mapping = DataPoint.display_map(client)
        with cls.display_writer(client, mapping, **kwargs) as writer:
            with closing(timeseries.live()) as live:
                for data_point in live:
                    writer.write_resources([data_point], mapping)

    return group


_list_options_docs = """
    Readings can be filtered by PORT and by START and END date and can
    be aggregated given an aggregation type and aggregation window size.

    Dates are given in ISO-8601 and may be one of the following forms:

    \b
    * YYYY-MM-DD - Example: 2016-05-05
    * YYYY-MM-DDTHH:MM:SSZ - Example: 2016-04-07T19:12:06Z

    Aggregations or bucketing of data can be done by specifying
    the size of each aggregation bucket using agg-size
    and one of the size specifiers.

    \b
    Examples: 1m, 2m, 5m, 10m, 30m, 1h, 1d

    How data-points are aggregated is indicated by a list of
    aggregation types using agg-type.

    \b
    Examples: min, max, avg

    For example, to aggregate min, max for a specific port 't' and
    aggregate on a daily basis use the following:

    \b
    --agg-type min,max --agg-size 1d --port t
"""


def list_options(page_size=20):
    """Standard options for retrieving timeseries readings. In addition it
    appends the documentation for these options to the caller.

    The common usecase is to use this function as a decorator for a
    command like:

    \b
    ...
    @timeseries.list_options()
    def list():
        ...
    """
    options = [
        click.option('--page-size', default=page_size,
                     help="the number of readings to get per request"),
        click.option('--port',
                     help="the port to filter readings on"),
        click.option('--start',
                     help="the start date to filter readings on"),
        click.option('--end',
                     help="the end date to filter readings on"),
        click.option('--agg-size',
                     help="the time window of the aggregation"),
        click.option('--agg-type',
                     help="the kinds of aggregations to perform"),
    ]

    def wrapper(func):
        func.__doc__ += _list_options_docs
        for option in reversed(options):
            func = option(func)
        return func
    return wrapper


_post_options_docs = """
    The given VALUE is inserted to the timeseries stream using the given PORT.

    The optional timestamp option allows fine grained control over the date of
    the reading and can be given in ISO8601 form:

    \b
    * YYYY-MM-DD - Example: 2016-05-05
    * YYYY-MM-DDTHH:MM:SSZ - Example: 2016-04-07T19:12:06Z
"""


def post_options():
    """Standard arguments and options for posting timeseries readings.
    """
    options = [
        click.argument('port'),
        click.argument('value', type=JSONParamType()),
        click.option('--timestamp', metavar='DATE',
                     help='the time of the reading'),
    ]

    def wrapper(func):
        func.__doc__ += _post_options_docs
        for option in reversed(options):
            func = option(func)
        return func
    return wrapper



from .__about__ import (
    __package_name__, __title__, __author__, __author_email__,
    __license__, __copyright__, __version__, __revision__,
    __url__,
)
from .resource import Resource, ResourceMeta
from .sensor import Sensor
from .element import Element
from .client import Client
from .user import User
from .label import Label
from .organization import Organization
from .timeseries import DataPoint
from .options import (
    device_sort_option,
    device_mac_option,
    ResourceParamType,
    JSONParamType
)

__all__ = (
    Resource, ResourceMeta,
    Sensor,
    Element,
    Client,
    User,
    Organization,
    Label,
    DataPoint,
    # Options
    device_sort_option,
    device_mac_option,
    ResourceParamType,
    JSONParamType,
    # Metadata attributes
    '__package_name__',
    '__title__',
    '__author__',
    '__author_email__',
    '__license__',
    '__copyright__',
    '__version__',
    '__revision__',
    '__url__',
)



from __future__ import unicode_literals
from helium import User
from operator import attrgetter


def display_map(cls, client):
    dict = super(User, cls).display_map(client)
    dict.update([
        ('name', attrgetter('name')),
    ])
    return dict

User.display_map = classmethod(display_map)



import click
from helium_commander import Client, Sensor
from helium_commander import device_mac_option, device_sort_option
from helium_commander.commands import timeseries


pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on physical or virtual sensors.
    """
    pass


@cli.command()
@click.argument('sensor', required=False)
@device_mac_option
@device_sort_option
@pass_client
def list(client, sensor, mac, **kwargs):
    """List sensors.

    Lists information for a given SENSOR or all sensors in the
    organization.

    """
    if sensor:
        sensors = [Sensor.lookup(client, sensor, mac=mac)]
    else:
        sensors = Sensor.all(client)
    Sensor.display(client, sensors, **kwargs)


@cli.command()
@click.option('--name', required=True,
              help="the name for the new virtual sensor")
@pass_client
def create(client, name):
    """Create a virtual sensor.

    Create a virtual sensor with a name.
    """
    sensor = Sensor.create(client, name=name)
    Sensor.display(client, [sensor])


@cli.command()
@click.argument('sensor')
@click.option('--name',
              help="the new name for the sensor")
@device_mac_option
@pass_client
def update(client, sensor, name, mac, **kwargs):
    """Updates the attributes of a sensor.

    Updates the attributes of a given SENSOR.
    """
    sensor = Sensor.lookup(client, sensor, mac=mac)
    sensor = sensor.update(name=name)
    Sensor.display(client, [sensor])


@cli.command()
@click.argument('sensor')
@device_mac_option
@pass_client
def delete(client, sensor, mac, **kwargs):
    """Delete a sensor.

    Deletes the SENSOR with the given id.
    """
    sensor = Sensor.lookup(client, sensor, mac=mac)
    result = sensor.delete()
    click.echo("Deleted" if result else result)

cli.add_command(timeseries.cli(Sensor))



from __future__ import unicode_literals
from helium import Sensor
from operator import attrgetter


def card_type(card_id, default):
    return {
        '2': 'blue',
        '5': 'green',
    }.get(card_id, default)


def display_map(cls, client):
    def _type(self):
        try:
            card = str(self.meta.card.get('id'))
            return card_type(card, card)
        except AttributeError:
            return None

    def _seen(self):
        try:
            return self.meta.last_seen
        except AttributeError:
            pass
        return None

    def _mac(self):
        try:
            return self.meta.mac
        except AttributeError:
            pass
        return None

    dict = super(Sensor, cls).display_map(client)
    dict.update([
        ('mac', _mac),
        ('type', _type),
        ('seen', _seen),
        ('name', attrgetter('name'))
    ])
    return dict

Sensor.display_map = classmethod(display_map)



import click
from helium_commander import Client, User


pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on the authorized user.
    """
    pass


@cli.command()
@pass_client
def list(client):
    """Lists user information for the current user.

    Lists information associated withe the user for the current api key.
    """
    user = User.singleton(client)
    User.display(client, [user])



import click
from helium_commander import Client, Element, Sensor
from helium_commander import device_mac_option, device_sort_option
from helium_commander.commands import timeseries


pass_client = click.make_pass_decorator(Client)


@click.group()
def cli():
    """Operations on physical or virtual sensors.
    """
    pass


@cli.command()
@click.argument('element', required=False)
@device_mac_option
@device_sort_option
@pass_client
def list(client, element, mac, **kwargs):
    """List elements.

    Lists information for a given ELEMENT or all elements in the
    organization.

    """
    if element:
        elements = [Element.lookup(client, element, mac=mac, include=[Sensor])]
    else:
        elements = Element.all(client, include=[Sensor])
    Element.display(client, elements, **kwargs)


@cli.command()
@click.argument('element')
@click.option('--name',
              help="the new name for the element")
@device_mac_option
@pass_client
def update(client, element, name, mac, **kwargs):
    """Updates the attributes of a element.

    Updates the attributes of a given ELEMENT.
    """
    element = Element.lookup(client, element, mac=mac, include=[Sensor])
    element = element.update(name=name)
    element = Element.lookup(client, element.id, mac=mac, include=[Sensor])
    Element.display(client, [element])


cli.add_command(timeseries.cli(Element))



"""The module that holds metadata about this library."""

from ._version import __version__, __revision__

__package_name__ = 'helium-commander'
__title__ = 'helium'
__author__ = 'Helium'
__author_email__ = 'hello@helium.com'
__license__ = 'BSD'
__copyright__ = 'Copyright 2016 Helium'
__url__ = 'http://github.com/helium/helium-commander'

__all__ = (
    '__package_name__', '__title__', '__author__', '__author_email__',
    '__license__', '__copyright__', '__version__', '__revision__',
    '__url__',
)
